Package: CEC
Type: Package
Title: Cross-Entropy Clustering
Version: 0.12.0
Date: 2026-08-30
Authors@R: c(
    person("Kamieniecki", "Konrad", email = "konrad.kamieniecki@alumni.uj.edu.pl",
      role = c("aut", "cph")),
    person("Spurek", "Przemyslaw", email = "przemyslaw.spurek@gmail.com",
      role = c("ctb")),
      person("Simon", "Garnier", email = "garnier@njit.edu", role = c("cre", "ctb", "cph"),
      comment = c(ORCID = "0000-0002-3886-3974"))
  )
Maintainer: Simon Garnier <garnier@njit.edu>
Description: Splits data into Gaussian type clusters using the Cross-Entropy 
    Clustering ('CEC') method. This method allows for the simultaneous use of 
    various types of Gaussian mixture models, for performing the reduction of 
    unnecessary clusters, and for discovering new clusters by splitting them. 
    'CEC' is based on the work of Spurek, P. and Tabor, J. (2014) 
    <doi:10.1016/j.patcog.2014.03.006>.
ByteCompile: true
URL: https://github.com/swarm-lab/cec, https://swarm-lab.github.io/cec/
Encoding: UTF-8
LazyData: true
Imports: graphics, methods, stats
NeedsCompilation: yes
License: GPL-3
Suggests: covr, testthat (>= 3.0.0)
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Packaged: 2026-08-30 23:14:28 UTC; simon
Author: Kamieniecki Konrad [aut, cph],
  Spurek Przemyslaw [ctb],
  Simon Garnier [cre, ctb, cph] (ORCID:
    <https://orcid.org/0000-0002-3886-3974>)
Repository: CRAN
Date/Publication: 2026-08-31 15:50:08 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-08-31 17:49:07 UTC; unix
Archs: CEC.so.dSYM
